/*=============================*\
How to use one GsSPRITE structure
to display more than one sprite.
Scaling and rotating sprites.
Moving Sprites.

			by Tom
\*=============================*/

#include <libps.h>
#include "MyPad.h"

#define OT_LENGTH		2
#define PACKETMAX		2048
#define PACKETMAX2		(PACKETMAX*24)
#define SCREEN_WIDTH	320
#define SCREEN_HEIGHT	256

#define SPRITE_ADDRESS 0x80090000

GsOT		WorldOT[2];	
GsOT_TAG	OTTags[2][1<<OT_LENGTH];
static PACKET packetArea[2][PACKETMAX2];
volatile u_char *bb0, *bb1;


void main(void);
void InitGame(void);
static u_long ReadPad(void);
void ProperInitialiseTexture(long TIMdata, GsIMAGE *imageInfo);


void main(void) {
	int activeBuff;
	u_long pad;
	GsSPRITE spriteHandler;
	GsIMAGE imageHandler;
	long rotate = 0, scale = ONE, x = 160, y = 128, i;		// define sprite's position and setup
	
	InitGame();

	ProperInitialiseTexture(SPRITE_ADDRESS, &imageHandler);
	
	spriteHandler.x = 160;
	spriteHandler.y = 128;
	
	switch(imageHandler.pmode) {
		case(8):
			spriteHandler.w = imageHandler.pw * 4;
			spriteHandler.attribute = 0;		// 4 bit image
			break;
		case(9):
			spriteHandler.w = imageHandler.pw * 2;
			spriteHandler.attribute = 1<<24;	// 8 bit image
			break;
		case(2):
			spriteHandler.w = imageHandler.pw;
			spriteHandler.attribute = 2<<24;	// 16 bit image
			break;
		default:
			printf("Sorry, image bit depth '%d' is not supported (is the image 24 bit?)\n", imageHandler.pmode);
			exit(0);
			break;
	}
	
	spriteHandler.h = imageHandler.ph;	
	spriteHandler.u = imageHandler.px % 64;
	spriteHandler.v = imageHandler.py % 256;
	spriteHandler.cx = imageHandler.cx;
	spriteHandler.cy = imageHandler.cy;
	spriteHandler.r = 128;
	spriteHandler.g = 128;
	spriteHandler.b = 128;
	spriteHandler.mx = 32;
	spriteHandler.my = 16;
	spriteHandler.tpage = GetTPage(imageHandler.pmode & 7, 0, imageHandler.px, imageHandler.py);
	
	do {
		pad = ReadPad();
		activeBuff = GsGetActiveBuff();
		GsSetWorkBase((PACKET *)packetArea[activeBuff]);
		GsClearOt(0, 0, &WorldOT[activeBuff]);
		
		
		// change the sprite's x and y coordinates
		if (pad & PADLleft)
			x-=4;
		if (pad & PADLright)
			x+=4;
		if (pad & PADLdown)
			y+=4;
		if (pad & PADLup)
			y-=4;
		
		// change the scaling values
		if (pad & PADLX)
			scale -= 32;
		if (pad & PADLT)
			scale += 32;
		
		// change the rotation values
		if (pad & PADLS)
			rotate -= ONE;
		if (pad & PADLO)
			rotate += ONE;
			
		// set the sprite rotation and scaling values
		spriteHandler.rotate = rotate;
		spriteHandler.scalex = scale;
		spriteHandler.scaley = scale;
		
		// draw 7 sprites in a column
		for (i=-3; i<4; i++) {
			spriteHandler.x = x;
			spriteHandler.y = y + i*32;
			GsSortSprite(&spriteHandler, &WorldOT[activeBuff], 0);
		}
		
		DrawSync(0);
		VSync(0);
		GsSwapDispBuff();
		
		GsSortClear(0, 0, 0, &WorldOT[activeBuff]);
		GsDrawOt(&WorldOT[activeBuff]);
	} while ((pad != PADLstart+PADLselect) && (pad != PADRselect+PADRstart));
}




void InitGame(void) {
	int i;
	
	SetVideoMode(MODE_PAL);
	GsInitGraph(SCREEN_WIDTH, SCREEN_HEIGHT, GsINTER|GsOFSGPU, 1, 0);
	GsDefDispBuff(0, 0, 0, SCREEN_HEIGHT);
	GsDISPENV.screen.h = 255;
	
	GetPadBuf(&bb0, &bb1);
	
	for (i = 0; i < 2; i++) {
		WorldOT[i].length = OT_LENGTH;
		WorldOT[i].org = OTTags[i];
		GsClearOt(0,0,&WorldOT[i]);
	}
}


// Generic sprite initialiser
void ProperInitialiseTexture(long TIMdata, GsIMAGE *imageInfo) {
	RECT rect;		// Frame buffer rectangular area

	// +4 because we need to skip head of TIM file
	// Put TIM data into GsIMAGE structure
	GsGetTimInfo((u_long *)(TIMdata+4), imageInfo);

	// define the area of the frame buffer to load the sprite into
	rect.x = imageInfo->px;
	rect.y = imageInfo->py;
	rect.w = imageInfo->pw;
	rect.h = imageInfo->ph;
	
	// Using rect data, load the pixel data held in the TIM file onto
	// the frame buffer
	LoadImage(&rect, imageInfo->pixel);
	// Wait for loading to finish
	DrawSync(0);

	// If image needs a CLUT, load it up 
	// (pmode: 8 or 9 ==> texture is 4-bit or 8-bit)
	if ((imageInfo->pmode>>3) & 0x01) {
		// define the area of the frame buffer to load the palette onto
		rect.x = imageInfo->cx;	
		rect.y = imageInfo->cy;	
		rect.w = imageInfo->cw;	
		rect.h = imageInfo->ch;	

		// Using rect data, load the palette onto the frame buffer
		LoadImage(&rect, imageInfo->clut);
		// wait for loading to finish
		DrawSync(0);
	}

}


u_long ReadPad(void) {
	return(~(*(bb0+3) | *(bb0+2) << 8 | *(bb1+3) << 16 | *(bb1+2) << 24));
}
